#include "my_parser.h"
#include "analyser_if.h"
#include <stdint.h>
#include <iostream>
#include <fstream>
#include <assert.h>

my_parser::my_parser(void):
  m_parser(XML_ParserCreate(NULL))
{
  if (!m_parser)
    {
      std::cout << "Couldn't allocate memory for parser" << std::endl ;
      exit(-1);
    }

  XML_SetUserData(m_parser,this);
  XML_SetElementHandler(m_parser,start, end);
}

void my_parser::parse(std::istream & p_stream)
{
  std::set<analyser_if*>::iterator l_iter = m_analysers.begin();
  std::set<analyser_if*>::iterator l_iter_end = m_analysers.end();
  while(l_iter != l_iter_end)
    {
      (*l_iter)->init();
      ++l_iter;
    }
//   std::ifstream l_file(m_name.c_str());
//   if(!l_file)
//     {
//       std::cout << "ERROR : Could not open file \"" << m_name << "\"" << std::endl ;
//       exit(-1);
//     }
  const uint32_t l_size = 10000;
  char l_buf[l_size];
  bool l_end = false;
  while(! (l_end = p_stream.eof()))
    {
      p_stream.read(l_buf,l_size);
      if (! XML_Parse(m_parser, l_buf,p_stream.gcount(),l_end))
	{
	  std::cout << "ERROR : Parse error at line " << XML_GetCurrentLineNumber(m_parser) << " :" << XML_ErrorString(XML_GetErrorCode(m_parser)) << std::endl;
	  exit(-1);
	}
    }
  std::cout << "PARSE SUCCESSFULL" << std::endl ;
}

void my_parser::start(void *data, const char *el, const char **attr)
{
  my_parser *l_parser = static_cast<my_parser*>(data);
  assert(l_parser);
  l_parser->analyse_start_element(el,attr);

}

void my_parser::end(void *data, const char *el)
{
  my_parser *l_parser = static_cast<my_parser*>(data);
  assert(l_parser);
  l_parser->analyse_end_element(el);
}

void my_parser::add_analyser(analyser_if & p_analyser)
{
  m_analysers.insert(&p_analyser);
}

void my_parser::analyse_start_element(const char *el, const char **attr)
{
  std::set<analyser_if*>::iterator l_iter = m_analysers.begin();
  std::set<analyser_if*>::iterator l_iter_end = m_analysers.end();
  while(l_iter != l_iter_end)
    {
      (*l_iter)->start_element(el);
      for (uint32_t i = 0; attr[i]; i += 2)
	{
	  (*l_iter)->get_attribute(attr[i],attr[i+1]);
	}
     ++l_iter;
    }
 
}

void my_parser::analyse_end_element(const char *el)
{
  std::set<analyser_if*>::iterator l_iter = m_analysers.begin();
  std::set<analyser_if*>::iterator l_iter_end = m_analysers.end();
  while(l_iter != l_iter_end)
    {
      (*l_iter)->end_element(el);
      ++l_iter;
    }
}

//EOF
