#include "osm_diff_watcher.h"
#include "url_reader.h"
#include "my_buffer.h"
#include "my_parser.h"
#include <sstream>
#include <iostream>
#include <fstream>
#include <assert.h>
#include <stdlib.h>
#include "gzstream.h"

osm_diff_watcher::osm_diff_watcher(void)
{
}

osm_diff_watcher::~osm_diff_watcher(void)
{
  url_reader::remove_instance();
}

void osm_diff_watcher::run(void)
{
  std::string l_prev_seq_number;
  uint32_t l_nb_iteration = 3 ;
  while(l_nb_iteration)
    {
      std::cout << l_nb_iteration << " remaining iterations" << std::endl ;
      uint32_t l_delay = 1;
      std::string l_seq_number;      
      do
	{
	  std::cout << "Wait for " << l_delay << " seconds" << std::endl ;
	  sleep(l_delay);
	  l_seq_number = get_sequence_number();     
	  if(l_delay < 30 ) l_delay *= 2;
	}while(l_prev_seq_number == l_seq_number);
      std::cout << "--> Sequence number = \"" << l_seq_number << "\"" << std::endl;
      
      std::string l_url_diff = get_url_diff(l_seq_number);
      std::cout << "Url of diff file \"" << l_url_diff << "\"" << std::endl ;
      dump_url(l_url_diff);
      parse_diff();

      //     std::cout << "Inscription date = \"" << get_user_inscription_date("quicky") << "\"" << std::endl ;
      l_prev_seq_number = l_seq_number;
      --l_nb_iteration;
    }
}

void osm_diff_watcher::parse_diff(void)
{
  igzstream l_input_file("tmp_diff.gz");
  m_analyser.init();
  my_parser l_parser;
  l_parser.add_analyser(m_analyser);
  l_parser.parse(l_input_file);
  //  std::string l_line;
  //  while(!getline(l_input_file,l_line).eof())
  //    {
  //      std::cout << l_line << std::endl ;
  //    }
  l_input_file.close();
}

void osm_diff_watcher::dump_url(const std::string & p_url)
{
  url_reader & l_url_reader = url_reader::get_instance();
  my_buffer l_buffer;
  l_url_reader.read_url(p_url.c_str(),l_buffer);
  std::ofstream l_output_file("tmp_diff.gz",std::ios::out | std::ios::binary);
  if(l_output_file==NULL)
    {
      std::cout << "ERROR : Unable to open output file \"tmp_diff.gz\"" << std::endl ;
      exit(EXIT_FAILURE);
    }
  l_output_file.write(l_buffer.get_data(),l_buffer.get_size());
  l_output_file.close();
}

std::string osm_diff_watcher::get_url_diff(const std::string & p_seq_number)
{
  std::string l_complete_seq_number = (p_seq_number.size() < 9 ? std::string(9 - p_seq_number.size(),'0') + p_seq_number : p_seq_number);
  // http://wiki.openstreetmap.org/wiki/Planet.osm/diffs
  std::string l_url_diff("http://planet.openstreetmap.org/minute-replicate/");
  l_url_diff += l_complete_seq_number.substr(0,3) + "/" + l_complete_seq_number.substr(3,3) + "/" + l_complete_seq_number.substr(6,3) + ".osc.gz";
  return l_url_diff;
}

std::string osm_diff_watcher::get_sequence_number(void)const
{
  std::string l_sequence_number;
  url_reader & l_url_reader = url_reader::get_instance();
  my_buffer l_buffer;
  l_url_reader.read_url("http://planet.openstreetmap.org/minute-replicate/state.txt",l_buffer);
  std::stringstream l_stream;
  l_stream << l_buffer.get_data();
  std::string l_line;
  while(!getline(l_stream,l_line).eof())
    {
      if(l_line.find("sequenceNumber") != std::string::npos)
	{
	  std::cout << "line = \"" << l_line << "\"" << std::endl ;
	  size_t l_begin = l_line.find("=");
	  l_sequence_number = l_line.substr(l_begin+1);
	}
    }
  assert(l_sequence_number != "");
  return l_sequence_number;
}


//EOF
